#!/usr/bin/env python
# -*- coding: UTF-8 -*-
##############################################################################
#
# PDFgui            by DANSE Diffraction group
#                   Simon J. L. Billinge
#                   (c) 2006 trustees of the Michigan State University.
#                   All rights reserved.
#
# File coded by:    Chris Farrow
#
# See AUTHORS.txt for a list of people who contributed.
# See LICENSE.txt for license information.
#
##############################################################################

# generated by wxGlade 0.4 on Wed Feb 22 21:20:24 2006

import wx
from diffpy.pdfgui.gui.pdfpanel import PDFPanel
from diffpy.pdfgui.gui.tooltips import datasetresultspanel as toolTips

class DataSetResultsPanel(wx.Panel, PDFPanel):
    def __init__(self, *args, **kwds):
        # begin wxGlade: DataSetResultsPanel.__init__
        kwds["style"] = wx.TAB_TRAVERSAL
        wx.Panel.__init__(self, *args, **kwds)
        self.sizer_panelname_staticbox = wx.StaticBox(self, -1, "")
        self.panelNameLabel = wx.StaticText(self, -1, "Data Set Results")
        self.labelScaleFactor = wx.StaticText(self, -1, "Scale Factor")
        self.textCtrlScaleFactor = wx.TextCtrl(self, -1, "", style=wx.TE_READONLY)
        self.labelQdamp = wx.StaticText(self, -1, "Qdamp")
        self.textCtrlQdamp = wx.TextCtrl(self, -1, "", style=wx.TE_READONLY)
        self.labelQbroad = wx.StaticText(self, -1, "Qbroad")
        self.textCtrlQbroad = wx.TextCtrl(self, -1, "", style=wx.TE_READONLY)
        self.buttonExport = wx.Button(self, wx.ID_OPEN, "Export PDF")

        self.__set_properties()
        self.__do_layout()

        self.Bind(wx.EVT_BUTTON, self.onExport, id=wx.ID_OPEN)
        # end wxGlade
        self.__customProperties()

    def __set_properties(self):
        # begin wxGlade: DataSetResultsPanel.__set_properties
        self.panelNameLabel.SetFont(wx.Font(18, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
        self.buttonExport.Hide()
        # end wxGlade
        self.setToolTips(toolTips)

    def __do_layout(self):
        # begin wxGlade: DataSetResultsPanel.__do_layout
        sizer_7 = wx.BoxSizer(wx.VERTICAL)
        grid_sizer_1 = wx.FlexGridSizer(4, 2, 5, 10)
        sizer_panelname = wx.StaticBoxSizer(self.sizer_panelname_staticbox, wx.HORIZONTAL)
        sizer_panelname.Add(self.panelNameLabel, 0, wx.LEFT|wx.RIGHT|wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_7.Add(sizer_panelname, 0, wx.LEFT|wx.RIGHT|wx.EXPAND, 5)
        grid_sizer_1.Add(self.labelScaleFactor, 0, wx.LEFT|wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 5)
        grid_sizer_1.Add(self.textCtrlScaleFactor, 0, wx.ALIGN_CENTER_VERTICAL, 20)
        grid_sizer_1.Add(self.labelQdamp, 0, wx.LEFT|wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 5)
        grid_sizer_1.Add(self.textCtrlQdamp, 0, wx.ALIGN_CENTER_VERTICAL, 20)
        grid_sizer_1.Add(self.labelQbroad, 0, wx.LEFT|wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 5)
        grid_sizer_1.Add(self.textCtrlQbroad, 0, wx.ALIGN_CENTER_VERTICAL, 20)
        sizer_7.Add(grid_sizer_1, 0, wx.ALL|wx.EXPAND, 5)
        sizer_7.Add(self.buttonExport, 0, wx.ALL|wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 5)
        self.SetSizer(sizer_7)
        sizer_7.Fit(self)
        # end wxGlade

    # USER CONFIGURATION CODE #################################################

    def __customProperties(self):
        self.results = {}
        self.ctrlMap = {'dscale'        :   'textCtrlScaleFactor',
                        'qdamp'         :   'textCtrlQdamp',
                        'qbroad'        :   'textCtrlQbroad',
                        }

        # Hide all of the pscale stuff until it is needed.
        #for i in range(10):
        #    self.pscaleGridSizer.Show(i, False)
        #self.pscaleGridSizer.Layout()
        return

    def setResultsData(self):
        """Set the values in the results panel.

        The values are taken from the results member dictionary.
        dscale      --  float
        qdamp       --  float
        qbroad      --  float
        """
        for name in self.ctrlMap:
            value = self.results.get(name, None)
            ctrlName = self.ctrlMap[name]
            textCtrl = getattr(self, ctrlName)
            if value is not None:
                textCtrl.SetValue(str(value))
            else:
                textCtrl.SetValue('')
        return

    # EVENT CODE #############################################################

    def onExport(self, event): # wxGlade: DataSetResultsPanel.<event_handler>
        print "Event handler `onExport' not implemented"
        event.Skip()

    # Methods overloaded from PDFPanel
    def refresh(self):
        """Refresh the panel."""

        # Enable the label widgets
        #selections = self.treeCtrlMain.GetSelections()
        #if selections:
        #    node = selections[0]
        #    phases = self.treeCtrlMain.GetPhases(node)
        #    for i in range(5):
        #        labelWidget = getattr(self, "pscaleLabel" + str(i+1))
        #        if i < len(phases):
        #            self.pscaleGridSizer.Show(2*i, True)
        #            self.pscaleGridSizer.Show(2*i+1, True)
        #            label = self.treeCtrlMain.GetItemText(phases[i])
        #            labelWidget.SetLabel(label)
        #        else:
        #            self.pscaleGridSizer.Show(2*i, False)
        #            self.pscaleGridSizer.Show(2*i+1, False)
        #    self.pscaleGridSizer.Layout()

        # Set the results data
        self.setResultsData()
        return



# end of class DataSetResultsPanel
